# -*- coding: UTF-8 -*-
from ConfigParser import ConfigParser
from ead2.lib.libead import EadKeyParser
from ead2.lib.error import *

class ServerParser(ConfigParser):
    """
    utilisé pour parser un fichier servers.ini
    (fichier qui definit les serveurs sur lequel un front-end peut se connecter
    """

    def __init__(self, backend_port, filename):
        ConfigParser.__init__(self)
        self.backend_port = backend_port
        self.filename = filename
        self.read(self.filename)
        
    def parse_file(self):
        """
        charge les serveurs depuis un fichier
        """
        self.read()

    def add_server(self, url, port, comment, key):
        """
        enregistre un nouveau serveur
        """
        servs = self.get_server()
        for serv in servs.values():
            if serv[0] == url and str(serv[1]) == str(port):
                raise ExistingServer
        cle = str(len(servs) + 1)
        servs[cle] = (url,port,comment,key)
        self.write_file(servs)

    def del_server(self, id_serv):
        """
        supprime un serveur
        """
        servs = self.get_server()
        try:
            del(servs[id_serv])
        except KeyError:
            raise UnknownServer
        server_keys = servs.keys()
        server_keys.sort()
        dico_tmp = {}
        #on va maintenant mettre a jour les indices des serveurs dans le dico
        #si on a les serveurs 1, 2 et 3, et qu'on supprime le 2, on veut que 3 devienne 2
        for key in server_keys:
            cle = str(server_keys.index(key)+1)
            dico_tmp[cle] = servs[key]
        self.write_file(dico_tmp)

    def get_server(self, id_serv = None):
        """
        renvoi les informations sur un serveur
        """
        self.read(self.filename)
        self._servers = {}
        for section in self.sections():
            URL = self.get(section, 'URL')
            try:
                port = eval(self.get(section, 'port'))
            except:
                port = self.backend_port
            try:
                comment = self.get(section, 'comment')
            except:
                comment = "Pas d'informations"
            try:
                key = self.get(section, 'key')
            except:
                key = ""
            self._servers[section] = (URL, port, comment, key)
        if id_serv != None:
            if not self._servers.has_key(id_serv):
                raise UnknownServer, "server %s not found" % id_serv
            else:
                return self._servers[id_serv]
        else:
            return self._servers

    def write_file(self, dict):
        """
        ecrit le fichier de serveurs
        """
        if self.sections() is not []:
            for section in self.sections():
                self.remove_section(section)
        for section, value in dict.items():
            URL, port, comment, key = value
            if not self.has_section(section):
                self.add_section(section)
            self.set(section, 'URL', URL)
            if eval(str(port)) == eval(str(self.backend_port)):
                port = 'BACKEND_LISTEN_PORT'
            self.set(section, 'port', port)
            self.set(section, 'comment', comment)
            self.set(section, 'key', key)
        try:
            fd = file(self.filename,'w')
            self.write(fd)
            fd.close()
        except:
            return False
        return True   
